* FMYSQL driver for FlagShip rel. 4.48+ and MySQL-Max rel. 3.23.44+
* (c) 2001 by Tore Van Grembergen <torevg@albatros-logistics.be>
* Licensed under LGPL (see README.fmysql for details)
*
* Compile: FlagShip fmysql.prg fmyrs.prg -c -m -na -I/usr/include/mysql
* Link:    FlagShip myapplic*.prg fmysql.o fmyrs.o -L/usr/lib/mysql -lmysqlclient -lgz
*
* or simple: make
*
* see also README.fmysql and mysqltest.prg for usage

CLASS TMYRS
    HIDDEN aDB          AS ARRAY
    HIDDEN aDBStruct    AS ARRAY
    HIDDEN nPos         AS INTVAR
    HIDDEN lBOF         AS LOGICAL
    HIDDEN lEOF         AS LOGICAL

METHOD INIT ( aDatabase, aDBStructure)  CLASS TMYRS
    nPos = 0
    IF aDatabase <> NIL
       aDB := aDatabase
       nPos := 1
    ENDIF
    IF aDBStructure <> NIL
       aDBStruct := aDBStructure
    ENDIF
    IF SELF:RECCOUNT() = 0
        lEof := .T.
        lBof := .T.
    ENDIF
RETURN Self

METHOD NoIvarGet(symIvarName) CLASS TMYRS
    LOCAL nFieldNr AS INTVAR
    LOCAL uRetVal  AS USUAL
    nFieldNr = Fieldnr(aDBStruct, symIvarName)
    IF nFieldNr > 0
        uRetVal := aDB[nPos,nFieldnr]
    ELSE
        MYBREAK("Field " + symIvarName + " does not exist !",PROCNAME(),PROCLINE())
    ENDIF
RETURN uRetVal


STATIC FUNCTION FIELDNR (aDBStruct, cIvarName)
RETURN ASCAN(aDBStruct,{|a|a[1]==cIvarName})

/*
***********************************************************************

    METHOD RECCOUNT ClASS TMYRS
    Retrieves  the number of  physical records in  the current recordset

***********************************************************************
*/


METHOD RECCOUNT ClASS TMYRS
    LOCAL nReccount AS INT
    nreccount = LEN(aDB)
RETURN nReccount

/*
***********************************************************************

    METHOD RECNO ClASS TMYRS
    Retrieves the current record number of the recordset.
    Returns: a  numeric value representing the current record number within
    the recordset.

***********************************************************************
*/

METHOD RECNO ClASS TMYRS
RETURN nPos

/*
***********************************************************************

    METHOD SKIP(nskip)  ClASS TMYRS
    Moves the  record pointer relative to the current pointer position.
    nskip specifies the  number of records to move the record pointer
    from  the  current position.  A  positive value  moves  the pointer
    forward,  while a  negative value  moves the pointer  backwards. If
    nskip is not specified, 1 is assumed.

***********************************************************************
*/

METHOD SKIP(nskip)  ClASS TMYRS
    IF nSkip = nil
        nSkip = 1
    ENDIF
    nPos := nPos + nSkip
    lBOF := .F.
    lEOF := .F.
    IF nPos < 1
       nPos := 1
       lBOF := .T.
    ENDIF
    IF nPos > LEN(aDB)
       nPos := LEN(aDB)
       lEOF := .T.
    ENDIF
RETURN nPos

/*
***********************************************************************

    METHOD FCOUNT() CLASS TMYRS
    Determines the number of fields in the current database file.

***********************************************************************
*/

METHOD FCOUNT() CLASS TMYRS
    LOCAL nFIELDS  AS INT
    nFields := LEN(aDBStruct)
RETURN nFIELDS

/*
***********************************************************************

    METHOD EOF() CLASS TMYRS
    Reports  an attempt to  move past the  end of  the current database
    file.

***********************************************************************
*/

METHOD EOF() CLASS TMYRS
return lEOF


/*
***********************************************************************

    METHOD BOF() CLASS TMYRS
    Detects  if there was an attempt to move  past the beginning of the
    current database file.

***********************************************************************
*/


METHOD BOF() CLASS TMYRS
return lBOF

/*
***********************************************************************

    METHOD GOTO(nRecNo) CLASS TMYRS
    Moves the database pointer to the specified record.
    nRecNo is  the  record  to  which  the  record  pointer is  to  be
    positioned.
    If  the  nRecNo is  out  of range,  the  database  pointer is
    positioned to the first or the last record.
***********************************************************************
*/


METHOD GOTO(nRecNo) CLASS TMYRS
    DO CASE
        CASE nRecNo = NIL
             nPos := 1
        CASE nrecNo < 1
             nPos := 1
        CASE nRecNo > LEN(aDB)
             nPos := LEN(aDB)
        OTHERWISE
            nPos := nRecNo
    ENDCASE
return

/*
***********************************************************************

    METHOD GOTOP() CLASS TMYRS
    Moves the database pointer to the first logical record.

***********************************************************************
*/

METHOD GOTOP() CLASS TMYRS
    nPos := 1
return

/*
***********************************************************************

    METHOD GOBOTTOM() CLASS TMYRS
    Moves the database pointer to the last logical record.

***********************************************************************
*/

METHOD GOBOTTOM() CLASS TMYRS
    nPos := LEN(aDB)
return

/*
***********************************************************************
    METHOD FIELDINFO(nTypeInfo,uField) CLASS TMYRS

    Retrieves   information  about  a  field.

    Arguments:
    nTypeInfo  specifies   the  required   type  of   the  field
    information, given as a constant (see "rddsys.fh") or a numeric value:

    Constant   Value   Ret       Returns
    -------------------------------------------------------------------
    DBS_NAME     1     retC      name of the field
    DBS_TYPE     2     retC      type of the field
    DBS_LEN      3     retN      length of the field
    DBS_DEC      4     retN      number of decimal places for the field

    uFieldis the  ordinal position of the  field in the record
    structure or  the  field  name as a string. Specifying  a  numeric  argument may
    perform slightly faster on databases with a large number of fields.

    Returns:  <retN>|<retC> is  the required  field information, or  NIL on error.

***********************************************************************
*/

METHOD FIELDINFO(nTypeInfo,uField) CLASS TMYRS
    LOCAL nFieldNr  AS INTVAR
    LOCAL uInfo     AS USUAL
    IF uField <> NIL
      Do CASE
         CASE VALTYPE(uField) = "N"
            IF uField <= LEN(aDBStruct) .AND. uField > 0
                nFieldNr = uField
            ENDIF
         CASE VALTYPE(uField) = "C"
            nFieldNr = FIELDNR(aDBStruct, uField)
            IF nFieldNr = 0
                nFieldNr := NIL
            ENDIF
      ENDCASE
      IF nFieldNr <> NIL
        uInfo= aDBStruct[nFieldNr][nTypeInfo]
      ENDIF
    ENDIF
return uInfo

/*
***********************************************************************

    METHOD DBSTRUCT() CLASS TMYRS

    Creates an array containing the structure of the current query.
    Returns a two-dimensional array containing  the structure of the
    current  query, whose  length is  equal to  the  number of
    fields  in the query.  Each element of  the array  is a subarray
    containing information for one field:

     Element  Type  Description     Usage
     [n,1]     C    Field name      1..10 chars
     [n,2]     C    Field type      C,N,D,L,M
     [n,3]     N    Field length    1..65535
     [n,4]     N    Deci places     0..18

***********************************************************************
*/

METHOD DBSTRUCT() CLASS TMYRS
return aDBStruct

/*
***********************************************************************

    METHOD FIELDGET(uField) CLASS TMYRS

    Retrieves  the value of a field using the ordinal position of the field
    in  query or the field name.

    Arguments:  uField is the ordinal position of the field in the
    record structure or the field  name. Specifying numeric argument may
    perform slightly faster on databases with a large number of fields.

    Returns the value  of the specified field.  If uField is out
    of range , the return value is NIL.

***********************************************************************
*/

METHOD FIELDGET(uField) CLASS TMYRS
    LOCAL nFieldNr  AS INTVAR
    LOCAL uRetVal   AS USUAL
    IF uField <> NIL
      Do CASE
         CASE VALTYPE(uField) = "N"
            IF uField <= LEN(aDBStruct) .AND. uField > 0
                nFieldNr = uField
            ENDIF
         CASE VALTYPE(uField) = "C"
            nFieldNr = FIELDNR(aDBStruct, uField)
            IF nFieldNr = 0
                nFieldNr := NIL
            ENDIF
      ENDCASE
      IF nFieldNr <> NIL
        uRetVal := aDB[nPos,nFieldnr]
      ENDIF
    ENDIF
RETURN uRetVal

/*
***********************************************************************
    METHOD FIELDLEN(uField) CLASS TMYRS

    Determines the size of a selected database field.

    Arguments: uField is the ordinal field number, or  the field name of
    the database in the current work area.

    Returns: the  length of  the  database field.  Equivalent  to the
    DBS_LEN subarray element of DBSTRUCT().
***********************************************************************
*/

METHOD FIELDLEN(uField) CLASS TMYRS
    LOCAL nFieldNr  AS INTVAR
    LOCAL uRetVal   AS USUAL
    IF uField <> NIL
      Do CASE
         CASE VALTYPE(uField) = "N"
            IF uField <= LEN(aDBStruct) .AND. uField > 0
                nFieldNr = uField
            ENDIF
         CASE VALTYPE(uField) = "C"
            nFieldNr = FIELDNR(aDBStruct, uField)
            IF nFieldNr = 0
                nFieldNr := NIL
            ENDIF
      ENDCASE
      IF nFieldNr <> NIL
        uRetVal := aDBStruct[nFieldNr,3]
      ENDIF
    ENDIF
RETURN uRetVal

/*
***********************************************************************
    METHOD FIELDDEC(uField) CLASS TMYRS

    Determines the number of decimals in a numeric database field.

    Arguments: uField is the ordinal field number, or  the field name of
    the database in the current work area.

    Returns: is  the number of  decimal places (0..17)  of numeric field,
        or  zero on non-numeric fields. Equivalent  to the DBS_DEC subarray
        element of DBSTRUCT().
***********************************************************************
*/

METHOD FIELDDEC(uField) CLASS TMYRS
    LOCAL nFieldNr  AS INTVAR
    LOCAL uRetVal   AS USUAL
    IF uField <> NIL
      Do CASE
         CASE VALTYPE(uField) = "N"
            IF uField <= LEN(aDBStruct) .AND. uField > 0
                nFieldNr = uField
            ENDIF
         CASE VALTYPE(uField) = "C"
            nFieldNr = FIELDNR(aDBStruct, uField)
            IF nFieldNr = 0
                nFieldNr := NIL
            ENDIF
      ENDCASE
      IF nFieldNr <> NIL
        uRetVal := aDBStruct[nFieldNr,4]
      ENDIF
    ENDIF
RETURN uRetVal
