* FMYSQL driver for FlagShip rel. 4.48+ and MySQL-Max rel. 3.23.44+
* or for Visual FlagShip 6 with MySQL rel. 5.0.x
* (c) 2001 by Tore Van Grembergen <torevg@albatros-logistics.be>
* Licensed under LGPL (see README.fmysql for details)
*
* Compile: FlagShip fmysql.prg fmyrs.prg -c -m -na -I/usr/include/mysql
* Link:    FlagShip myapplic*.prg fmysql.o fmyrs.o -L/usr/lib/mysql -lmysqlclient -lgz
*
* or simply "make"
*
* see also README.fmysql and mysqltest.prg for usage

#Cinline
/* #include <FSextend.h> */
#include <FSopenc.h>
#include <mysql.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
MYSQL		conn;			  /* dbd: data base descriptor						 */
MYSQL_RES	*rs;			  /* the resultset from the database				 */
MYSQL_FIELD *myfields;		  /* the fielddefinition from the resultset 		 */

static int	IsOpen = 0; 	  /* only one open connection is allowed per session */
char		errors[4000];	  /* char array to put errormessages in 			 */
static int	fieldcount; 	  /* Number of fields in the resultset				 */
int 		sqlerrorno =0;	  /* SQL Error number								 */
char		sqlerror[4000];   /* SQL Error Description							 */


void prerror(char *string)
{
	FILE *fp;
	long time(), now;
	char errfile[100];
	time( &now );
	sprintf(errfile, "./mysql%s.err", getenv("USER"));

	fp = fopen(errfile, "a");
	if(fp!= NULL)
	{ fprintf(fp, "%s ", getenv("USER") );
	  fprintf(fp, "%s ", ctime(&now));
	  fprintf(fp, "Error: %s\n", string);
	  fclose( fp );
	}
	return;
}

void prsql(char *string)
{
	FILE *fp;
	long time(), now;
	char errfile[100];
	time( &now );
	sprintf(errfile, "./mysql%s.sql", getenv("USER"));

	fp = fopen(errfile, "a");
	if(fp!= NULL)
	{ fprintf(fp, "%s ", getenv("USER") );
	  fprintf(fp, "%s ", ctime(&now));
	  fprintf(fp, "SQL : %s\n", string);
	  fclose( fp );
	}
	return;
}
void SetMysqlError(MYSQL *mysql)
{ sqlerrorno = mysql_errno(mysql);
  sprintf(sqlerror,"MYSQL ERROR : %s",mysql_error(mysql));
}

void ResetMysqlError(MYSQL *mysql)
{sqlerrorno = 0;
 sprintf(sqlerror,"");
}


int verify(MYSQL *mysql)
{
  if(mysql_errno(mysql))
  {
	sprintf(errors,"Error: %s",mysql_error(mysql));
	prerror(errors);
	SetMysqlError(mysql);
	return 1;
  }
  else
  {ResetMysqlError(mysql);
  }
  return 0;
}
#endCinline

#include "error.fh"
#include "fmyrs.fh"

/*
***********************************************************************

	FUNCTION MYDBOPEN(cDBNAME, cHOST, cUSER, cPASSWORD)
	attempts to establish a connection to a MySQL database engine
	running on host. MYDBOPEN(cDBNAME, cHOST, cUSER, cPASSWORD)
	must complete successfully before you can execute any
	of the other FUNCTIONs.
	Returns .T. if login was succesfull

***********************************************************************
*/


FUNCTION MYDBOPEN(cDBNAME, cHOST, cUSER, cPASSWORD)
	LOCAL lConnected := .F. AS LOGICAL

#Cinline
{

   char *dbname, *host, *user, *password, *unixsock;
   int port, flags;


   dbname = VAR_CHR(VAR_NAME_LOCPAR(cdbname));
   host = VAR_CHR(VAR_NAME_LOCPAR(chost));
   user = VAR_CHR(VAR_NAME_LOCPAR(cuser));
   password = VAR_CHR(VAR_NAME_LOCPAR(cpassword));

   port = 3306;
   unixsock = NULL;
   flags = 0;

  /* only one open connection is allowed per session */
  if (IsOpen)
  {
	prerror("Attempt to open a database twice.");
	printf("Attempt to open a database twice.");
	goto done;
  }

  if(!mysql_init(&conn))
  {
	prerror("Cannot create the conection structure.");
	printf("Cannot create the conection structure.");
	goto done;
  }

  if(!mysql_real_connect(&conn,host,user,password,dbname,port,unixsock,flags))
  {
	if(verify(&conn))
	{
	sprintf(errors, "User %s cannot connect to %s host, database %s" , user,host,dbname);
	prerror(errors);
	SET_VAR_LOG(VAR_NAME_LOCAL(lconnected),0);
	IsOpen=0;
	}
  }
  else
  {
	IsOpen=1;
	SET_VAR_LOG(VAR_NAME_LOCAL(lconnected),1);
  }
done:
  ;
}
#endCinline
	IF lConnected = .F.
		MYBREAK(MYERROR(),PROCNAME(), PROCLINE())
	ENDIF
RETURN	lConnected

/*
***********************************************************************

	FUNCTION MYDBCLOSE()
	Closes a previously opened connection.
	MYDBCLOSE() also deallocates the connection handle
	pointed to by mysql

***********************************************************************
*/


FUNCTION MYDBCLOSE()
#Cinline
{
  if(IsOpen==0)
  {
	sprintf(errors, "There isn't actually any database open");
	prerror(errors);
  }
  mysql_close(&conn);
  IsOpen = 0;

}
#endCinline
RETURN NIL

/*
***********************************************************************

	STATIC FUNCTION MYRSCLOSE()
	Frees the memory allocated for a result set
	When you are done with a result set, you must free the memory it uses.
	Must be called after every MYQUERY(cSQL,aParams)

***********************************************************************
*/


STATIC FUNCTION MYRSCLOSE()
#Cinline
{
  mysql_free_result(rs); /* frees the memory asociated with result */
  rs = NULL;
}
#endCinline
RETURN NIL

/*
***********************************************************************

	FUNCTION MYEXEC(cSQL,aParams)
	executes a query that does not return any rows.
	FUNCTION returns the affected rows by the command.
	parameters : cSQL = DELETE, INSERT, UPDATE, ...
	aParams := twodimensinal array containing  the values of the parameters
	[1] : name of the parameter
	[2] : value of the parameter


***********************************************************************
*/

FUNCTION MYEXEC(cSQL, aParams)
	LOCAL nRows 	AS INT
	cSQL := MYPARSEPARAMS(cSQL, aParams)
#Cinline
{
  char *cmd;
  cmd = VAR_CHR(VAR_NAME_LOCPAR(csql));

  #ifdef _WRITESQL_
	prsql(cmd);
  #endif

  ResetMysqlError(&conn);
  mysql_query(&conn,cmd);
  if (verify(&conn))
  { sprintf(errors, "Error executing : User %s executed : %s", getenv("USER"),cmd);
	prerror(errors);
  }

  nrows = mysql_affected_rows(&conn);

}
#endCinline
	IF MYERRORNO() != 0
		MYBREAK(MYERROR(),PROCNAME(), PROCLINE())
	ENDIF
RETURN nRows

/*
***********************************************************************

	FUNCTION MYQUERY(cSQL, aParams)
	Executes the SQL query pointed to by the string cSQL.
	Executes a query that returns rows.
	The query must consist of a single SQL statement.
	You should not add a terminating semicolon (`;') or \g to the statement.
	FUNCTION sets the global rs and the fieldcount variable
	parameters : cSQL = Select, show ...
	aParams := twodimensinal array containing  the values of the parameters
	[1] : name of the parameter
	[2] : value of the parameter

	if an mysql error occurred the FUNCTION returns -1.

	STATIC FUNCTION : can not be called from outside this file !!

***********************************************************************
*/

STATIC FUNCTION MYQUERY(CSQL, aParams)
	LOCAL nRetValue AS INT
	cSQL = MYPARSEPARAMS(cSQL, aPARAMS)
#Cinline
{
  char *sql;
  sql = VAR_CHR(VAR_NAME_LOCPAR(csql));

  #ifdef _WRITESQL_
	prsql(sql);
  #endif

  ResetMysqlError(&conn);

  mysql_query(&conn,sql);
  if (verify(&conn))
  { sprintf(errors, "Error executing : User %s executed : %s", getenv("USER"),sql);
	prerror(errors);
  }

  if(sqlerrorno==0)
  { rs=mysql_use_result(&conn);
	if (verify(&conn))
	{ sprintf(errors, "ERR_CANNOT_RECOVERY_RESULT : User %s executed : %s", getenv("USER"),sql);
	  prerror(errors);
	}
	if(sqlerrorno==0)
	{fieldcount=mysql_num_fields(rs);
	 myfields = mysql_fetch_fields(rs);
	}
  }
  else
  {mysql_free_result(rs); /* frees the memory asociated with result */
   rs = NULL;
  }

}
#endCinline
	IF MYERRORNO() != 0
		MYBREAK(MYERROR(),PROCNAME(), PROCLINE())
	ENDIF
RETURN nRetValue

/*
***********************************************************************

	FUNCTION MYQTODBF(cSQL,aParams)
	executes a query that returns rows.
	sets the result in a dbf file.
	FUNCTION returns the filename of the dbf file
	parameters : cSQL = Select, show ...
	aParams := twodimensinal array containing  the values of the parameters
	[1] : name of the parameter
	[2] : value of the parameter


***********************************************************************
*/

FUNCTION MYQTODBF(cSQL,aParams)
  LOCAL cTempName	AS STRING
  LOCAL cAlias		AS STRING
  LOCAL adbstruct	AS USUAL
  LOCAL aTupple 	AS USUAL
  MYQUERY(CSQL,aParams)
  cTempname = MYTEMPNAME() + ".dbf"
  cAlias := MYALIAS(cTempname)
  adbstruct :=	MYDBSTRUCT()
  DBCREATE(cTempname,adbstruct)
  USE (cTempname) ALIAS (cAlias) EXCLUSIVE NEW
	DO WHILE .T.
	   aTupple := MYGETDBTUPLE(adbstruct)
	   IF aTupple = NIL
			EXIT
	   ENDIF
	   APPEND BLANK
	   FIELDPUTARR(aTupple)
	ENDDO
  (cAlias)->(DBCOMMIT())
  IF (cAlias)->(USED())
	  (cAlias)->(DBCLOSEAREA())
  ENDIF
  MYRSCLOSE()
RETURN cTempname

/*
***********************************************************************

	FUNCTION MYQTOTXT(cSQL,aParams)
	executes a query that returns rows.
	sets the result in a txt file.
	FUNCTION returns the filename of the txt file
	parameters : cSQL = Select, show ...
	aParams := twodimensinal array containing  the values of the parameters
	[1] : name of the parameter
	[2] : value of the parameter

***********************************************************************
*/

FUNCTION MYQTOTXT(cSQL,aParams)
  LOCAL cTempName AS STRING
  LOCAL nField	  AS INT
  LOCAL aTupple   AS USUAL
  LOCAL fhandle   AS INT
  LOCAL cTupple   AS STRING
  MYQUERY(CSQL,aParams)
  cTempname = MYTEMPNAME() +".txt"
  fhandle := fcreate(cTempname)
  IF fhandle != -1
	DO WHILE .T.
	   aTupple := MYGETTUPLE()
	   IF aTupple = NIL
		EXIT
	   ENDIF
	   cTupple := ""
	   FOR nField = 1 TO MYFCOUNT()
		   IF !(empty(cTupple))
			   cTupple := cTupple + " | "
		   ENDIF
		   cTupple := cTupple + aTupple[nField]
	   NEXT
	   cTupple := cTupple + chr(10)
	   fwrite(fhandle,cTupple)
	ENDDO
  ENDIF
  fclose(fhandle)
  MYRSCLOSE()
RETURN cTempName

/*
***********************************************************************

	FUNCTION MYQTOMEM(cSQL, aParams)
	executes a query that returns rows.
	sets the result in a array
	FUNCTION returns the array, every element in the array is a string
	parameters : cSQL = Select, show ...
	aParams := twodimensinal array containing  the values of the parameters
	[1] : name of the parameter
	[2] : value of the parameter

***********************************************************************
*/

FUNCTION MYQTOMEM(cSQL,aParams)
  LOCAL aTupple AS USUAL
  LOCAL aDB 	AS ARRAY
  MYQUERY(CSQL,aParams)
  DO WHILE .T.
	 aTupple := MYGETTUPLE()
	 IF aTupple = NIL
		  EXIT
	 ENDIF
	 AADD(aDB,aTupple)
  ENDDO
  MYRSCLOSE()
RETURN aDB

/*
***********************************************************************

	FUNCTION MYQTODBMEM(cSQL,aParams)
	executes a query that returns rows.
	sets the result in a array
	FUNCTION returns the array, every element in the array is typecasted
	parameters : cSQL = Select, show ...
	aParams := twodimensinal array containing  the values of the parameters
	[1] : name of the parameter
	[2] : value of the parameter

***********************************************************************
*/

FUNCTION MYQTODBMEM(cSQL,aParams)
  LOCAL adbstruct AS ARRAY
  LOCAL aTupple   AS USUAL
  LOCAL aDB 	  AS ARRAY
  MYQUERY(CSQL,aParams)
  adbstruct :=	MYDBSTRUCT()
  DO WHILE .T.
	 aTupple := MYGETDBTUPLE(adbstruct)
	 IF aTupple = NIL
		  EXIT
	 ENDIF
	 AADD(aDB,aTupple)
  ENDDO
  MYRSCLOSE()
RETURN aDB

/*
***********************************************************************

	FUNCTION MYQTORS(cSQL,aParams)
	executes a query that returns a TMYRS object.
	This is an resultset with some methods on.
	a fieldvalue can be read by executing rs:fieldname
	parameters : cSQL = Select, show ...
	aParams := twodimensinal array containing  the values of the parameters
	[1] : name of the parameter
	[2] : value of the parameter

***********************************************************************
*/

FUNCTION MYQTORS(cSQL,aParams)
  LOCAL adbstruct AS ARRAY
  LOCAL aTupple   AS USUAL
  LOCAL aDB 	  AS ARRAY
  LOCAL rs		  AS TMYRS
  MYQUERY(CSQL,aParams)
  adbstruct :=	MYDBSTRUCT()
  DO WHILE .T.
	 aTupple := MYGETDBTUPLE(adbstruct)
	 IF aTupple = NIL
		  EXIT
	 ENDIF
	 AADD(aDB,aTupple)
  ENDDO
  rs := TMYRSNew(aDB,adbstruct)
  MYRSCLOSE()
RETURN rs

/*
***********************************************************************
	STATIC FUNCTION MYTEMPNAME()
	returns the  generated,  random and  unique  file  name, without
	extension.
	if MYTEMPDIR is set as an environment parameter then
	the temporary file is created in MYTEMPDIR
	otherwise the  temporary file is created in the tmp dir
	set by the system

***********************************************************************
*/
/*#define STANDARD_FLAGSHIP*/  /* comment-out for own tempnam() */

STATIC FUNCTION MYTEMPNAME()
  LOCAL cTempname AS STRING

#ifdef STANDARD_FLAGSHIP
  LOCAL cTmpDir := GetEnv("MYTEMPDIR")
	cTempname := TempFileName(ALLTRIM(cTmpDir),"MYSQL")
#else
  #Cinline
  {
	 char *tempdir;
	 tempdir = getenv("MYTEMPDIR");
	 if (tempdir == NULL) tempdir="";
	 SET_VAR_CHR(VAR_NAME_LOCAL(ctempname),tempnam(tempdir,"MYSQL"));

	 SET_VAR_LOWER(VAR_NAME_LOCAL(ctempname));
  }
  #endCinline
#ENDif

RETURN cTempname

/*
***********************************************************************
	STATIC FUNCTION MYFCOUNT()
	Determines the number of fields in the current query
	It is a  numeric value,  representing the  number  of database
	fields in the current query.
***********************************************************************
*/

STATIC FUNCTION MYFCOUNT()
  LOCAL nFIELDS  AS INT
#Cinline
{
	nfields = fieldcount;
}
#endCinline
RETURN nFIELDS

/*
***********************************************************************
	STATIC FUNCTION MYGETTUPLE()
	Retrieves the next row of a result set.
	Returns NIL when there are no more rows to retrieve
	Returns the row as an array.  Every element in the array is a string

***********************************************************************
*/

STATIC FUNCTION MYGETTUPLE()
	LOCAL aTupple	AS USUAL
#Cinline
{
	FSvar *arow;
	int j;
	MYSQL_ROW	tuple;			  /* the current accessed row	*/
	if(rs != NULL)
	{ if(tuple=mysql_fetch_row(rs))
	  {VAR_NEW_ARRAY(arow,fieldcount);
	   for(j=0;j<fieldcount;j++)
		  { SET_VAR_CHR(VAR_ARRELEM(arow,j+1),tuple[j]);}
	   SET_VAR_COPY(VAR_NAME_LOCAL(atupple),arow);
	  }
	}
}
#endCinline
RETURN aTupple

/*
***********************************************************************
	STATIC FUNCTION MYGETDBTUPLE()
	Retrieves the next row of a result set.
	Returns NIL when there are no more rows to retrieve
	Returns the row as an array.  Every element in the array is typecasted

***********************************************************************
*/

STATIC FUNCTION MYGETDBTUPLE(aDbstruct)
	LOCAL aTupple	AS USUAL
	LOCAL nField	AS INT
	LOCAL cType 	AS STRING
	LOCAL nFieldlen AS INT
	LOCAL cValue	AS STRING
	LOCAL lValue	AS LOGICAL
	aTupple := MYGETTUPLE()
	IF aTupple != NIL
	  FOR nField = 1 TO MYFCOUNT()
		  cType :=	aDbstruct[nfield,2]
		  nFieldlen := aDbstruct[nfield,3]
		  DO CASE
			CASE (cType = "D")
					aTupple[nField] = STOD(STRTRAN(aTupple[nField],"-",""))
			CASE (cType = "C")
					aTupple[nField] = PADR(aTupple[nField],nFieldlen)
			CASE (cType = "N")
					aTupple[nField] = VAL(aTupple[nField])
/*			CASE (cType = "M")
				 nothing to DO */
			CASE (cType = "L")
					cValue := aTupple[nField]
					DO CASE
					  CASE cValue == "1"
						lValue := .T.
					  CASE cValue == "0"
						lValue := .F.
					ENDCASE
					aTupple[nField] := lValue
		  ENDCASE
	  NEXT
	 ENDIF
RETURN aTupple

/*
***********************************************************************

	STATIC FUNCTION MYDBSTRUCT()
	Creates an array containing the structure of the current query.
	Returns a two-dimensional array containing	the structure of the
	current  query, whose  length is  equal to	the  number of
	fields	in the query.  Each element of	the array  is a subarray
	containing information for one field:

	 Element  Type	Description 	Usage
	 [n,1]	   C	Field name		1..10 chars
	 [n,2]	   C	Field type		C,N,D,L,M
	 [n,3]	   N	Field length	1..65535
	 [n,4]	   N	Deci places 	0..18

***********************************************************************
*/

STATIC FUNCTION MYDBSTRUCT()
	LOCAL aDbstruct AS USUAL
#Cinline
{
  FSvar *afield, *arecordstruct;
  unsigned int i;
  char myfieldname[250];

  VAR_NEW_ARRAY(arecordstruct,fieldcount);

  for(i = 0; i < (unsigned int)fieldcount; i++)
  { VAR_NEW_ARRAY(afield,4);

	  /* define fieldname */

	if(strlen(myfields[i].name)>10)
	 {strncpy(myfieldname,myfields[i].name,10);}
	 else
	 {strcpy(myfieldname,myfields[i].name);}
	 myfieldname[10]='\0';

	SET_VAR_CHR(VAR_ARRELEM(afield,1),myfieldname);

	/* define length */

	SET_VAR_INT(VAR_ARRELEM(afield,3),myfields[i].length);

	/* define fieldtype */

	switch(myfields[i].type)
	{
	  case FIELD_TYPE_TINY:
	  case FIELD_TYPE_SHORT:
	  case FIELD_TYPE_LONG:
	  case FIELD_TYPE_LONGLONG:
	  case FIELD_TYPE_INT24:
	  case FIELD_TYPE_ENUM:
	  case FIELD_TYPE_SET:
	  case FIELD_TYPE_FLOAT:
	  case FIELD_TYPE_DECIMAL:
	  case FIELD_TYPE_DOUBLE:
		 SET_VAR_CHR(VAR_ARRELEM(afield,2),"N");
		 if(myfields[i].length > 19)
		 {SET_VAR_INT(VAR_ARRELEM(afield,3),19);}
		 else
		 {SET_VAR_INT(VAR_ARRELEM(afield,3),myfields[i].length);}
		 break;
	  case FIELD_TYPE_TIMESTAMP:
	  case FIELD_TYPE_TIME:
	  case FIELD_TYPE_DATETIME:
	  case FIELD_TYPE_TINY_BLOB:
	  case FIELD_TYPE_STRING:
	  case FIELD_TYPE_YEAR:
	  case FIELD_TYPE_VAR_STRING:
		  SET_VAR_CHR(VAR_ARRELEM(afield,2),"C");
		  if(myfields[i].length > 65535)
		  {SET_VAR_CHR(VAR_ARRELEM(afield,2),"M");
		   SET_VAR_INT(VAR_ARRELEM(afield,3),10);}
		  break;
	  case FIELD_TYPE_DATE:
		  SET_VAR_CHR(VAR_ARRELEM(afield,2),"D");
		  SET_VAR_INT(VAR_ARRELEM(afield,3),8);
		  break;
	  case FIELD_TYPE_NULL:
	  case FIELD_TYPE_NEWDATE:
	  case FIELD_TYPE_MEDIUM_BLOB:
	  case FIELD_TYPE_LONG_BLOB:
	  case FIELD_TYPE_BLOB:
		  SET_VAR_CHR(VAR_ARRELEM(afield,2),"M");
		  SET_VAR_INT(VAR_ARRELEM(afield,3),10);
		  break;
	}

	if(myfields[i].decimals > 8)
	  SET_VAR_INT(VAR_ARRELEM(afield,4),8);
	else
	  SET_VAR_INT(VAR_ARRELEM(afield,4),myfields[i].decimals);

	/* copy fieldstructure into recordstructure */

	SET_VAR_COPY(VAR_ARRELEM(arecordstruct,i+1),afield);

  }

  SET_VAR_COPY(VAR_NAME_LOCAL(adbstruct),arecordstruct);
}
#endCinline

RETURN aDbstruct


/*
***********************************************************************

	FUNCTION MYINSERTID()
	Returns the ID generated for an AUTO_INCREMENT column by the previous query.
	Use this FUNCTION after you have performed an INSERT query into
	a table that contains an AUTO_INCREMENT field.
	Note that MYINSERTID()returns 0 if the previous query does
	not generate an AUTO_INCREMENT value.
	if you need to save the value for later, be sure to call
	MYINSERTID() immediately after the query that generates the value.


***********************************************************************
*/

FUNCTION MYINSERTID()
  LOCAL nInsertid AS INT
#Cinline
{
	ninsertid = mysql_insert_id(&conn);
}
#endCinline
RETURN nInsertid

/*
***********************************************************************
	STATIC FUNCTION MYERRORNO()
	FUNCTION MYERRORNO() returns the error code for the most recently
	invoked API FUNCTION that can succeed or fail.
	A return value of zero means that no error occurred.

***********************************************************************
*/

STATIC FUNCTION MYERRORNO()
  LOCAL nErrorno AS INT
#Cinline
{ 
	nerrorno = sqlerrorno;
}
#endCinline
RETURN nErrorno

/*
***********************************************************************
	STATIC FUNCTION MYERROR()
	FUNCTION MYERROR()	returns the error message for the most recently
	invoked API FUNCTION that can succeed or fail.
	An empty string ("") is returned if no error occurred.

***********************************************************************
*/

STATIC FUNCTION MYERROR()
  LOCAL cError AS STRING
#Cinline
{ 
	SET_VAR_CHR(VAR_NAME_LOCAL(cerror),sqlerror);
}
#endCinline
RETURN cError

/*
***********************************************************************

	FUNCTION MYSTAT()
	Returns a character string containing information similar to that
	provided by the mysqladmin status command.
	This includes uptime in seconds and the number of running threads,
	questions, reloads, and open tables

***********************************************************************
*/

FUNCTION MYSTAT()
  LOCAL cMystat AS STRING
#Cinline
{
  char *mystat;
  mystat = mysql_stat(&conn);
  SET_VAR_CHR(VAR_NAME_LOCAL(cmystat),mystat);
}
#endCinline
RETURN cMystat

/*
***********************************************************************

	FUNCTION MYBEGIN()
	By default, MySQL runs in autocommit mode. This means that as soon as
	you execute an update, MySQL will store the update on disk
	if you want to switch from AUTOCOMMIT mode for one series of statements,
	you can use the MYBEGIN() FUNCTION.
	After this you must use MYCOMMIT() to store your changes to disk or
	MYROLLBACK() if you want to ignore the changes you have made since
	the beginning of your transaction.

***********************************************************************
*/

FUNCTION MYBEGIN()
	MYEXEC("BEGIN")
RETURN nil

/*
***********************************************************************

	FUNCTION MYCOMMIT()
	use MYCOMMIT to store your changes since the beginning of
	your transaction  to disk

***********************************************************************
*/

FUNCTION MYCOMMIT()
	MYEXEC("COMMIT")
RETURN nil

/*
***********************************************************************

	FUNCTION MYROLLBACK()
	use MYROLLBACK() if you want to ignore the changes you have made
	since the beginning of your transaction.


***********************************************************************
*/

FUNCTION MYROLLBACK()
	MYEXEC("ROLLBACK")
RETURN nil

/*
***********************************************************************

	FUNCTION MYPARSEPARAMS(cSQL, aParams)
	Parses the Sql statement.
	Replaces the parameters in the sql statement with there value.
	(select field1 from table1 where field2 = :value1)

***********************************************************************
*/

STATIC FUNCTION MYPARSEPARAMS(cSQL, aParams)
	LOCAL nTeller		AS INT
	LOCAL cParamHolder	AS STRING
	LOCAL cParam		AS STRING
	IF PCOUNT() = 2 .AND. aParams <> NIL
		FOR nTeller = 1 TO LEN(aParams)
			cParamHolder = ":" + aParams[nteller,1]
			IF aParams[nteller,2] != NIL
			  DO CASE
				  CASE VALTYPE(aParams[nteller,2]) = "C"
					  cParam := MYESCSTRING(ALLTRIM(aParams[nteller,2]))
				  CASE VALTYPE(aParams[nteller,2]) = "D"
					  cParam := "'" + TRANSFORM(VAL(DTOS(aParams[nteller,2])),"9999-99-99") + "'"
				  CASE VALTYPE(aParams[nteller,2]) = "F"
					  cParam := ALLTRIM(STR(aParams[nteller,2]))
				  CASE VALTYPE(aParams[nteller,2]) = "I"
					  cParam := ALLTRIM(STR(aParams[nteller,2]))
				  CASE VALTYPE(aParams[nteller,2]) = "M"
					  cParam := MYESCSTRING(aParams[nteller,2])
				  CASE VALTYPE(aParams[nteller,2]) = "N"
					  cParam := ALLTRIM(STR(aParams[nteller,2]))
			  ENDCASE
			ELSE
			  cParam := "NULL"
			ENDIF
			cSQL = STRTRAN(cSQL,cParamHolder,cParam)
		NEXT
	ENDIF
RETURN cSQL


/*
***********************************************************************

	FUNCTION MYESCSTRING(cSQLParam)
	Escapes the Sql Parameter.
	(') -> (\')

***********************************************************************
*/

STATIC FUNCTION MYESCSTRING(cSQLParam)
	cSQLParam := ALLTRIM(cSQLParam)
	IF (EMPTY(cSQLParam)) = .F.
		cSQLParam = STRTRAN(cSQLParam,"\","\\")
		cSQLParam = STRTRAN(cSQLParam,"'","\'")
		cSQLParam = STRTRAN(cSQLParam,'"','\"')
		cSQLParam = STRTRAN(cSQLParam,CHR(0),'\"')
		cSQLParam := "'" + cSQLParam + "'"
	ELSE
		cSQLParam := "NULL"
	ENDIF
RETURN cSQLParam

/*
***********************************************************************

		 FUNCTION MYBREAK(cDescription,cProcname , nProcline)  when encountered, terminates the
		 sequence by branching the execution  to  the  first  statement
		 following	the  corresponding RECOVER statement  if one is specified,
		 or the matching ENDSEQUENCE statement.  if executed outside
		 of the  BEGIN..END structure, runtime errors occur.

		 MYBREAK(cDescription) returns oError which is an error object, to
		 the <var> of the RECOVER USING clause.

***********************************************************************
*/

FUNCTION MYBREAK(cDescription, cProcname , nProcline)
	LOCAL oError AS ERROR

	oError := ERROR{}

	oError:CANRETRY := .F.
	oError:CANSUBSTITUTE := .T.
	oError:DESCRIPTION := cDescription
	oError:GENCODE := 0
	oError:OPERATION := "Error in FUNCTION " + ALLTRIM(cProcname) + " at line " + ALLTRIM(STR(nProcline))
	oError:OSCODE := DOSERROR()
	oError:SEVERITY := ES_IOERR
	oError:SUBCODE := MYERRORNO()
	oError:SUBSYSTEM := "fmysql"
	oError:TRIES := 0

	IF ISBEGSEQ()
		BREAK(oError)
	ELSE
		MYALERT(oError)
	ENDIF
RETURN

/*
	FUNCTION MYALERT(oError, aOptions,cColor)
	Displays a simple modal dialog box with the errormessage build up by MYBREAK()

	The message is shown  centered	in	the  alert	box.

	oError is an ERROR object.

	aOptions defines a	list of possible responses to the dialog box. If
	not specified, a single "OK" option is presented.

	cColor defines	the color  specification.

	MYALERT() returns a numeric value  indicating which option	was chosen. If
	the ESC key is pressed, zero is returned.

*/

FUNCTION MYALERT(oError, aOptions,cColor)
	LOCAl cErrorString	AS STRING
	LOCAL nKeuze		AS INTVAR

	cErrorString := "MYSQL ERROR;"
	cErrorString := cErrorString + "***********;"
	cErrorString := cErrorString + ";"
	cErrorString := cErrorString + ALLTRIM(oError:DESCRIPTION) + ";"
	cErrorString := cErrorString + "Errornumber = " + ALLTRIM(oError:SUBCODE) + ";"
	cErrorString := cErrorString + oError:OPERATION

	nKeuze := ALERT(cErrorString,aOptions,cColor)
RETURN nkeuze

FUNCTION MYALIAS(cFilename)
	LOCAL cAlias		AS STRING
	LOCAL nPos			AS INTVAR
	cFilename := LOWER(cFilename)
	cAlias := STRTRAN(cFilename,".dbf","")
	cAlias := STRTRAN(cAlias,"mysql","")
	nPos := RAT("/",cAlias)
	cAlias := SUBSTR(cAlias,nPos+1)
RETURN cALIAS
